import mariadb
import json
from pathlib import Path
from time import perf_counter
from pprint import pprint
from itertools import chain, repeat

conn = mariadb.connect(
       user="root",
       password="root",
       host="localhost",
       port=3306,
       database='imdb'
    )

conn.autocommit = False

c = conn.cursor()


c.execute('SELECT title_id, ordering FROM t_aka')
t_aka_PKs = set(c.fetchall())


with open('enums.json', encoding='utf-8') as enums_f:
    enums = json.load(enums_f)
for key in enums:  # transform to lookup table for ids: enums['genre']['Comedy'] => genre.id
    enums[key] = {name: i for i, name in enumerate(enums[key], 1)}


def idstrip(idstr, prefix_len=2):
    return int(idstr.strip()[prefix_len:])

def id_helper(parts, column, col_name, prefix_len):
    return idstrip(parts[column[col_name]], prefix_len)

def value_helper(parts, column, col_name, cls):
    cv = parts[column[col_name]].strip()
    return cls(cv) if cv != r'\N' else None

def enum_helper(parts, column, col_name, enum_name):
    v = value_helper(parts, column, col_name, str)
    return enums[enum_name][v] if v is not None else None

# valhelp and idhelp uses the parts and column directly from current scope
valhelp = lambda col_name, cls=str: value_helper(parts, column, col_name, cls)
idhelp = lambda  col_name, prefix_len=2: id_helper(parts, column, col_name, prefix_len)
enmhelp = lambda col_name, enum_name: enum_helper(parts, column, col_name, enum_name)


# c.execute('TRUNCATE TABLE t_aka') # (only for repeated cell running in notebook)
# c.execute('TRUNCATE TABLE t_aka_has_type')
t1 = perf_counter()
with open(Path('data') / 'title.akas.tsv', encoding='utf-8') as f:
    column = {}
    for i, p in enumerate(f.readline().strip().split('\t')):
        column[p.strip()] = i
    for li, line in enumerate(f):
        parts = line.strip().split('\t')
        id_ = idhelp('titleId')  # ommit prefix
        ordering = valhelp('ordering', int)

        title = valhelp('title')
        region = enmhelp('region', 'region')
        language = enmhelp('language', 'language')
        attributes = valhelp('attributes')
        is_orig = valhelp('isOriginalTitle', int)
        if (id_, ordering) not in t_aka_PKs:
            c.execute('INSERT INTO t_aka (title_id, ordering, title, region_id, language_id, attributes, is_original) VALUES (?,?,?,?,?,?,?)',
                      (id_, ordering, title, region, language, attributes, is_orig))

        # there is a quirk in data, some include \u0002 instead of comma
        phase1 = parts[column['types']].strip().split(',')
        phase2 = [p.split('\x02') for p in phase1]
        glist = list(chain.from_iterable(phase2))

        try:
            if len(glist) and glist[0] != '\\N':
                values = tuple(chain.from_iterable((id_, ordering, enums['t_aka_type'][p]) for p in glist if len(p)))
                qmarks = ','.join(repeat('(?,?,?)', len(values) // 3))
                c.execute(f'INSERT INTO t_aka_has_type (title_id, ordering, type_id) VALUES {qmarks}', values)
        except mariadb.IntegrityError:
            pass

        if li % 10000 == 0:
            conn.commit()
    conn.commit()
print(perf_counter() - t1, 's')
